@@ -36,7 +36,7 @@ except ImportError:
     gzip = None
     GZIP_BASE = object
 
-__version__ = '1.0.2'
+__version__ = '1.0.3'
 
 
 class FakeShutdownEvent(object):
@@ -148,8 +148,7 @@ except ImportError:
 
         Taken from https://pypi.python.org/pypi/six/
 
-        Modified to set encoding to UTF-8 if not set when stdout may not be
-        a tty such as when piping to head
+        Modified to set encoding to UTF-8 always
         """
         fp = kwargs.pop("file", sys.stdout)
         if fp is None:
@@ -159,7 +158,7 @@ except ImportError:
             if not isinstance(data, basestring):
                 data = str(data)
             # If the file has an encoding, encode unicode with it.
-            encoding = fp.encoding or 'UTF-8'  # Diverges for notty
+            encoding = 'UTF-8'  # Always trust UTF-8 for output
             if (isinstance(fp, file) and
                     isinstance(data, unicode) and
                     encoding is not None):
@@ -667,7 +666,10 @@ class SpeedtestResults(object):
             'upload': self.upload,
             'ping': self.ping,
             'server': self.server,
-            'timestamp': self.timestamp
+            'timestamp': self.timestamp,
+            'bytes_sent': self.bytes_sent,
+            'bytes_received': self.bytes_received,
+            'share': self._share,
         }
 
     def csv(self, delimiter=','):
@@ -1190,6 +1192,12 @@ def parse_args():
         parser.add_argument = parser.add_option
     except AttributeError:
         pass
+    parser.add_argument('--no-download', dest='download', default=True,
+                        action='store_const', const=False,
+                        help='Do not perform download test')
+    parser.add_argument('--no-upload', dest='upload', default=True,
+                        action='store_const', const=False,
+                        help='Do not perform upload test')
     parser.add_argument('--bytes', dest='units', action='store_const',
                         const=('byte', 8), default=('bit', 1),
                         help='Display values in bytes instead of bits. Does '
@@ -1197,7 +1205,7 @@ def parse_args():
                              'output from --json or --csv')
     parser.add_argument('--share', action='store_true',
                         help='Generate and provide a URL to the speedtest.net '
-                             'share results image')
+                             'share results image, not displayed with --csv')
     parser.add_argument('--simple', action='store_true', default=False,
                         help='Suppress verbose output, only show basic '
                              'information')
@@ -1286,11 +1294,15 @@ def shell():
     if args.version:
         version()
 
+    if not args.download and not args.upload:
+        raise SpeedtestCLIError('Cannot supply both --no-download and '
+                                '--no-upload')
+
     if args.csv_header:
         csv_header()
 
     if len(args.csv_delimiter) != 1:
-        raise SystemExit('--csv-delimiter must be a single character')
+        raise SpeedtestCLIError('--csv-delimiter must be a single character')
 
     validate_optional_args(args)
 
@@ -1318,6 +1330,11 @@ def shell():
     else:
         quiet = False
 
+    if args.csv or args.json:
+        machine_format = True
+    else:
+        machine_format = False
+
     # Don't set a callback if we are running quietly
     if quiet or debug:
         callback = do_nothing
@@ -1381,21 +1398,27 @@ def shell():
     printer('Hosted by %(sponsor)s (%(name)s) [%(d)0.2f km]: '
             '%(latency)s ms' % results.server, quiet)
 
-    printer('Testing download speed', quiet,
-            end=('', '\n')[bool(debug)])
-    speedtest.download(callback=callback)
-    printer('Download: %0.2f M%s/s' %
-            ((results.download / 1000.0 / 1000.0) / args.units[1],
-             args.units[0]),
-            quiet)
+    if args.download:
+        printer('Testing download speed', quiet,
+                end=('', '\n')[bool(debug)])
+        speedtest.download(callback=callback)
+        printer('Download: %0.2f M%s/s' %
+                ((results.download / 1000.0 / 1000.0) / args.units[1],
+                 args.units[0]),
+                quiet)
+    else:
+        printer('Skipping download test')
 
-    printer('Testing upload speed', quiet,
-            end=('', '\n')[bool(debug)])
-    speedtest.upload(callback=callback)
-    printer('Upload: %0.2f M%s/s' %
-            ((results.upload / 1000.0 / 1000.0) / args.units[1],
-             args.units[0]),
-            quiet)
+    if args.upload:
+        printer('Testing upload speed', quiet,
+                end=('', '\n')[bool(debug)])
+        speedtest.upload(callback=callback)
+        printer('Upload: %0.2f M%s/s' %
+                ((results.upload / 1000.0 / 1000.0) / args.units[1],
+                 args.units[0]),
+                quiet)
+    else:
+        printer('Skipping upload test')
 
     if args.simple:
         print_('Ping: %s ms\nDownload: %0.2f M%s/s\nUpload: %0.2f M%s/s' %
@@ -1407,10 +1430,12 @@ def shell():
     elif args.csv:
         print_(results.csv(delimiter=args.csv_delimiter))
     elif args.json:
+        if args.share:
+            results.share()
         print_(results.json())
 
-    if args.share:
-        printer('Share results: %s' % results.share(), quiet)
+    if args.share and not machine_format:
+        printer('Share results: %s' % results.share())
 
 
 def main():
